import { IBaseProps } from "office-ui-fabric-react/lib/Utilities";

/**
 * File encoding values.
 */
export const enum FileEncoding {
    Unknown,
    Binary,
    ASCII,
    UTF8,
    UTF32_BE,
    UTF32_LE,
    UTF16_BE,
    UTF16_LE
}

/**
 * Specifies how a file is uploaded to the server.
 */
export const enum FileInputContentType {
    /**
     * The file is base64-encoded.
     */
    Base64EncodedText,
    /**
     * The file is uploaded as a string.
     */
    RawText, 
    /**
     * The file is uploaded directly.
     */
    RawFile
}

/**
 * Event data passed to FileInput update events.
 */
export interface FileInputUpdateEventData {
    /**
     * Indicates that the files are still being read.
     */
    loading: boolean;
    /**
     * The files.
     */
    files: FileInputData[];
}

/**
 * File result from files uploaded to the FileInput.
 */
export interface FileInputResult {
    /**
     * The name of the file.
     */
    name: string;
    /**
     * The file type.
     */
    type: string;
    /**
     * The size of the file, in bytes.
     */
    size: number;
    /**
     * The last-modified timestamp.
     */
    lastModifiedDate: Date;
    /**
     * For Base64EncodedText or RawText, the content of the file.
     */
    content?: string;
    /**
     * The file encoding.
     */
    encoding?: FileEncoding;
    /**
     * For RawFile, the actual file.
     */
    file?: File;
    /**
     * A friendly status description, e.g. "2MB"
     */
    statusText: string;
    /**
     * Indicates whether there was a problem loading the file.
     */
    isError: boolean;
}

/**
 * Encapsulates a file operation.
 */
export interface FileInputData {
    /**
     * The FileReader that reads the file.
     */
    fileReader: FileReader;
    /**
     * The result of the file operation.
     */
    result: FileInputResult;
}

/**
 * Context for the limit message handler on why the upload limit was reached.
 */
export interface FileInputControlLimitEventData {
    /**
     * Size of the file that was too large, in bytes.
     */
    fileSize?: number;
    /**
     * Total size of all files, if the size was too large, in bytes.
     */
    totalSize?: number;
    /**
     * Number of files the user attempted to upload, if it was capped by maximumNumberOfFiles in the options.
     */
    fileCount?: number;
}

export interface IFileInputProps extends IBaseProps {
    /**
     * The maximum number of files the component can load.
     */
    maximumNumberOfFiles?: number;
    /**
     * File extensions allowed by the component.
     */
    allowedFileExtensions?: string[];
    /**
     * Indicates whether filenames are case-sensitive.
     */
    fileNamesCaseSensitive?: boolean;
    /**
     * The maximum total size, in bytes, of all files loaded by the control.
     */
    maximumTotalFileSize?: number;
    /**
     * The maximum size, in bytes, of any file loaded by the control.
     */
    maximumSingleFileSize?: number;
    /**
     * The type (base64, raw string, raw bytes) of the file result.
     */
    resultContentType?: FileInputContentType;
    /**
     * Indicates whether the component should attempt to detect the file encoding.
     */
    detectEncoding?: boolean;
    /**
     * Callback executed when the file operation progresses.
     * @param updateEvent The event
     */
    updateHandler: (updateEvent: FileInputUpdateEventData) => void;
    /**
     * Callback executed whenever a user bumps into the limit of the file upload control.
     * @param currentMessage The message provided by the control itself.
     * @param limitEvent The context data about why the limit was hit.
     * @returns A message that will be shown to the user in place of currentMessage.
     */
    limitMessageFormatter?: (currentMessage: string, limitEvent: FileInputControlLimitEventData) => string;
}