import * as React from 'react';
import { IFilter } from '../../Utilities/Filter';
import { IHubViewState } from '../../Utilities/HubViewState';
import { IObservableArray } from '../../Utilities/Observable';
import { IFilterBar, IFilterBarProps } from '../../Components/FilterBar/FilterBar.Props';
import {
    IPivotBar,
    IPivotRenderingModeOptions,
    IVssPivotBarItemProvider,
    PivotBarFocusItem,
} from '../../Components/PivotBar/PivotBar.Props';
import { IPivotBarAction, IPivotBarViewAction } from '../../Components/PivotBar/PivotBarAction.Props';
import { ISplitterProps } from '../../Components/Splitter/Splitter.Props';

export interface IHub {
    /**
     * Sets focus to the hub. If a particular focusItem is provided, will filter the focus() call
     * down to the appropriate sub-component.
     */
    focus(focusItem?: PivotBarFocusItem): void;
    
    /**
     * Gives access to the PivotBar component being hosted by this hub component.
     */
    pivotBar: IPivotBar;
}

export const enum ScrollableRegion {
    /**
     * The enitre hub will be scrollable.
     */
    Hub,
    /**
     * Just the pivot bar will be scrollable.
     */
    PivotBar
}

export interface IHubProps {

    /**
     * Optional callback to access the IHub interface. Use this instead of ref for accessing
     * the public methods and properties of the component.
     */
    componentRef?: (component: IHub) => void;

    /**
     * The state of the current view of the hub (selected pivot, filter, view options, etc.)
     */
    hubViewState: IHubViewState;

    /**
     * Optional root classname for the hub element.
     */
    className?: string;

    /**
     * Set of commands available on every pivot. The global commands are ordered before pivot-specific commands.
     */
    commands?: IPivotBarAction[] | IObservableArray<IPivotBarAction>;

    /**
     * Set of far actions available on every pivot. Global far actions are ordered after pivot-specific far actions.
     */
    viewActions?: IPivotBarViewAction[] | IObservableArray<IPivotBarAction>;

    /**
     * List of PivotBar item providers
     */
    pivotProviders?: IVssPivotBarItemProvider[];

    /**
     * Optional callback that is invoked after the pivot bar has been rendered.
     */
    onPivotBarRenderComplete?: () => void;

    /**
     * Called when the user requests to change the pivot. If this method doesn't return true, the
     * change will be blocked.
     */
    onBeforePivotChange?: (ev: React.MouseEvent<HTMLElement> | React.KeyboardEvent<HTMLElement>, pivotKey: string, url?: string) => boolean;

    /**
     * If true, don't show the Full Screen view action
     */
    hideFullScreenToggle?: boolean;

    /**
     * If true, use the box-shadow styling for the header. This should only be set by Hubs that use a non-white background for their content.
     */
    useShadowStyle?: boolean;

    /**
     * If true, hide the HubHeader, the pivot items (tabs), commands, view actions, and filters. Only show the
     * selected pivot item's content.
     */
    hideHeader?: boolean;

    /**
     * Optional method to render this pivot's FilterBar. A FilterBar can be supplied as a child element of this PivotBar, or be returned by this delegate.
     */
    onRenderFilterBar?: () => React.ReactElement<IFilterBarProps>;

    /**
     * Optional value providing explicit control over where the filter bar is displayed:
     * If true, show the filter bar inline with the pivot commands and view actions.
     * If false, add a toggle button in the view actions which displays the filter bar in a new row
     * If undefined (default), shows inline only if there is a single FilterBar item and it is a Keyword filter item.
     */
    showFilterBarInline?: boolean;

    /**
     * Explicitly sets the visibility of the pivots (tabs). If false, render the commands and view actions inline with any inner (non-pivot) content.
     * If this prop is undefined (default), then the pivots will be shown if there is more than one pivot item.
     */
    showPivots?: boolean;

    /**
     * Optional pivot rendering mode options. By default, pivots will be rendered horizontally.
     */
    pivotRenderingModeOptions?: IPivotRenderingModeOptions;

    /**
     * CSS class name to append to pivot bar
     */
    pivotBarClassName?: string;

    /**
     * CSS class name to append to pivot bar content
     */
    pivotBarContentClassName?: string;

    /**
     * When specified, the hub will render a splitter with secondary content in the near pane and the header/pivot bar in the far pane.
     */
    onRenderSecondaryContent?: () => JSX.Element;

    /**
     * Props to pass to the splitter component when leftContent is specified.
     */
    splitterProps?: ISplitterProps;

    /**
     * Optional value to determine if the entire hub or just the pivot bar content is scrollable.
     * @defaultvalue ScrollableRegion.PivotBar
     */
    scrollableRegion?: ScrollableRegion;
}