import { SelectionMode } from "office-ui-fabric-react/lib/Selection";

import { IItemIndicatorProps } from "../../ItemIndicator";
import { IFilterBarItemProps } from '../../FilterBar';
import { IVssIconProps } from "../../VssIcon";
import { FilterOperatorType } from "../../Utilities/Filter";
import { IObservableValue } from "../../Utilities/Observable";

export interface IPickListItem {

    /**
     * The value to display for this item
     */
    name: string;

    /**
     * Key used by the PickList to determine selection
     */
    key: string;
    
    /*
    * Optional aria-label to use for the item. If none is provided, the name will
    * be used by default.
    */
    ariaLabel?: string;

    /*
    * Optional tooltip to use for the item that appears on hover. If none is provided, the name will
    * be used by default.
    */
    tooltipText?: string;

    /**
     * Group which the item should belong to
     */
    groupKey?: string;
    
    /*
    * Optional url for the item to use as a hyperlink for the item's entry.
    */
    href?: string;

    /*
     * Optional props for the icon to show next to the item's name.
     */
    iconProps?: IVssIconProps;
    
    /**
     * Optional prop to disable the item
     */
    disabled?: boolean | (() => boolean);
}

export interface IPickListIndicator {
    /**
     * Returns an indicator for the given item.
     */
    getItemIndicator: (item: any) => IItemIndicatorProps | null | undefined;
}

export interface IPickListAction {

    /**
     * The text to display for this action button
     */
    name: string;

    /**
     * The optional icon to display next to this action button.
     */
    iconProps?: IVssIconProps;

    /**
     * The handler to invoke when the action is clicked 
     */
    onClick?: (ev: any) => void;
    
    /*
    * Optional url for the action to use as a hyperlink for the item's entry.
    */
    href?: string;

    /**
     * Whether the action should be rendered as disabled
     */
    disabled?: boolean;

    /**
    * Optional aria label for this action button
    */
    ariaLabel?: string;

    /*
    * Optional tooltip to use for the action that appears on hover.
    */
    tooltipText?: string;
}

export interface IPickListGroup {

    /**
     * Unique key for the group
     */
    key: string;

    /**
     * An optional name to use as the group header
     */
    name?: string;

    /**
     * Custom render method for the group header
     */
    onRenderGroupHeader?: (group: IPickListGroup) => JSX.Element;

    /**
     * If true, display a loading spinner in place of the group.
     */
    isLoading?: boolean;

    /**
     * Optional custom loading message to display for the group rather than "Loading..."
     */
    loadingMessage?: string;
}

export interface IPickListOperator {

    /**
     * Text to display for the operator
     */
    name: string;

    /**
     * Key to use for the operator
     */
    key: string;
}

export interface IPickListSelection {
    /**
     * Currently selected items in the pick list
     */
    selectedItems: any[];

    /**
     * Currently selected operator in the pick list
     */
    selectedOperatorKey?: FilterOperatorType;
}

export interface IPickListProps {

    /**
     * Optional callback to access the IPickListDropdown interface. Use this instead of ref for accessing
     * the public methods and properties of the component.
     */
    componentRef?: (component: IPickList) => void;

    /**
     * If provided, additional class name to provide on the root element.
     */
    className?: string;

    /**
     * Controls how/if the list manages selection.
     */
    selectionMode?: SelectionMode;

    /**
     * Callback that is called when the selection of the PickList changes - contains the entire
     * set of currently selected keys.
     * 
     * @returns True to prevent the default handler for the event that triggered the change.
     */
    onSelectionChanged?: (selection: IPickListSelection) => void | boolean;

    /**
     * The items to display
     */
    items?: any[] | Promise<any[]>;

    /**
     * Returns a PickListItem for the specified item
     */
    getListItem?: (item: any) => IPickListItem;

    /**
     * Indicators to draw on the right side of each item
     */
    indicators?: IPickListIndicator[];

    /**
     * Any items in the list which should be initially selected.
     * This is only a default value, so changes to this prop will
     * have no effect on the state of the component have the initial
     * render. Only use this if this PickList is an uncontrolled component.
     * Otherwise, use selectedItems.
     */
    initiallySelectedItems?: any[];

    /**
     * The selected items in the pick list. Only use this if this PickList is a
     * controlled component. Otherwise, use initiallySelectedItems.
     */
    selectedItems?: any[];

    /**
     * Whether or not the search box should be shown for the PickList
     */
    isSearchable?: boolean;

    /**
     * Custom text to show when a promise is supplied to the items prop but it is not yet fulfilled. Defaults to "Loading..."
     */
    itemsLoadingText?: string;

    /**
     * The minimum number of items in the PickList for the search box to appear.
     * Default is 8.
     */
    minItemsForSearchBox?: number;

    /**
     * Placeholder text used in the search TextField
     */
    searchTextPlaceholder?: string;

    /**
     * Text to display when results are being loaded
     */
    searchResultsLoadingText?: string;

    /**
     * Text to display when results are being loaded
     */
    searchResultsGroupName?: string;

    /**
     * Text to display when there are no results (based on the current search criteria) 
     */
    searchNoResultsText?: string;

    /**
     * Aria label to be associated with search box input 
     */
    searchBoxAriaLabel?: string;

    /**
     * Optional method to perform a custom search. By default a simple string contains is applied to the names in the current set of items. 
     */
    onSearch?: (searchText: string, items: any[]) => any[] | Promise<any[]>;

    /**
     * Optional callback to fetch a list of actions to render after all items and groups,
     * based on items in the list
     */
    getActions?: (items: IPickListItem[]) => IPickListAction[];

    /**
     * Groups of items to render.
     */
    groups?: IPickListGroup[];

    /**
     * Whether or not focusing an item in the PickList should cause it to be selected.
     */
    selectOnFocus?: boolean;

    /**
     * Text or a component representing a rich text message to display when there are no items in the list
     */
    noItemsText?: string | JSX.Element;

    /**
     * List of operators to appear in the pick list
     */
    operators?: IPickListOperator[];

    /**
     * Key for the operator which should be selected
     */
    selectedOperatorKey?: string;

    /**
     * Whether or not Select all should appear as an item in the PickList.
     * Only applicable if a multiple selection picklist.
     */
    showSelectAll?: boolean;

    /**
     * Forces a width for the PickList. 
     */
    width?: number;

    /**
     * Optional callback to determine whether the PickList should be rendered in full or virtualized.
     * Default behavior is virtualized which is suggested.
     */
    shouldVirtualize?: (itemsToRender: IPickListItem[]) => boolean;

    /**
     * Optional callback indicating that the user has tabbed out of the PickList's focus zone - 
     * a shift tab from the first focusable element, or a tab from the last focusable element.
     */
    onFocusLost?: () => void;

    /**
     * Optional callback which custom rendering of pick list element.
     */
    onRenderItem?: (item: IPickListItem) => JSX.Element;

    /**
     * Optional unique DOM element id. Added for accessibility.
     */
    id?: string;
}

export interface IPickList {
    /**
     * Clears the selection of the PickList, such that there are no selected items.
     * Only use this if this PickList is an uncontrolled component, otherwise, just
     * pass in new selectedItems as props.
     */
    clearSelection(): void;

    /**
     * Sets browser focus in the PickList. If the list contains a filter TextField,
     * focus will be set there. Otherwise, focus will be placed on the first list item.
     */
    focus(): boolean;

    /**
     * Re-render the PickList
     */
    forceUpdate(): void;

    /**
     * Reset the selectedItems back to the initiallySelectedItems 
     * Only use this if this PickList is an uncontrolled component, otherwise, just
     * pass in new selectedItems as props.
     */
    reset(): void;

    /**
     * Selects all items in the PickList. Only works when selectionMode is multiple and no onSearch callback
     * is defined.
     * Only use this if this PickList is an uncontrolled component, otherwise, just
     * pass in new selectedItems as props.
     */
    selectAll(): void;
}

export interface IPickListDropdownProps extends IPickListProps {
    /**
     * Optional callback to access the IPickListDropdown interface. Use this instead of ref for accessing
     * the public methods and properties of the component.
     */
    componentRef?: (component: IPickListDropdown) => void;

    /**
     * Provides items to the PickList - called when the PickList is about to
     * be rendered, not when the Dropdown itself is rendered.
     */
    getPickListItems: () => any[] | Promise<any[]>;

    /**
     * Text to show for the dropdown button when there are no selected item
     */
    placeholder?: string;

    /**
     * ClassName to pass along to the PickList component itself. We need this because
     * the picklist dropdown element is not an actual parent of the PickList (it's in a
     * callout).
     */
    pickListClassName?: string;
    
    /**
     * Indicators that will be displayed on the right side of the selected item (on the component that you click to open the dropdown)
     */
    selectedItemIndicators?: IPickListIndicator[];

    /**
     * Formattable string that contains an ariaLabel to use for the component.
     * If there is an {0} argument available, will append the text of the dropdown,
     * which is the currently selected items, if any.
     */
    ariaLabelFormat?: string;

    /**
     * If true, then the PickListDropdown will enter a disabled state, where it cannot be interacted with
     * and will be styled to indicate this state.
     */
    disabled?: boolean;

    /**
     * If false, dropdown will dismiss when the target element scrolls.
     * @defaultValue true
     */
    preventDismissOnScroll?: boolean;
    
    /**
     * The tab index for the dropdown element
     */
    tabIndex?: number;

    /**
     * If true, do not show the icon on the root selected item (on the component that you click to open the dropdown)
     */
    hideSelectedItemIcon?: boolean;

    /**
     * Overrides the default (chevron) dropdown icon
     */
    dropdownIconProps?: IVssIconProps;
}

export interface IPickListDropdown {
    /**
     * Sets browser focus in the PickListDropdown. The focus will be given to the title element.
     */
    focus(): void;

    /**
     * Re-render the PickListDropdown
     */
    forceUpdate(): void;
}

export interface IPickListFilterBarItemProps extends IPickListDropdownProps, IFilterBarItemProps {
    /**
     * Whether or not the and/or operators should be shown in the pick list.
     */
    showOrAndOperators?: boolean;

    /**
     * Don't show the Clear button.
     */
    hideClearButton?: boolean;
}

/**
 * Provides the items and metadata necessary to render an item picker
 */
export interface IItemPickerProvider<T = {}> {

    /**
     * The currently selected item in the picker.
     */
    selectedItem?: T;

    /**
     * Indicators that will be displayed on the right side of each item in the pick list dropdown
     */
    dropdownIndicators?: IPickListIndicator[];

    /**
     * Indicators that will be displayed on the right side of the selected item (on the component that you click to open the dropdown)
     */
    selectedItemIndicators?: IPickListIndicator[];

    /**
     * Callback that is called when the selection of the PickList changes
     * 
     * @returns True to prevent the default handler for the event that triggered the change.
     */
    onSelectedItemChanged?: (selectedItem: T) => void | boolean;

    /**
     * The items to display
     */
    getItems: () => T[] | Promise<T[]>;

    /**
     * Returns a PickListItem for the specified item
     */
    getListItem?: (item: T) => IPickListItem;

    /**
     * Whether or not the search box should be shown for the PickList
     */
    isSearchable?: boolean;

    /**
     * Placeholder text used in the search TextField
     */
    searchTextPlaceholder?: string;

    /**
     * Text to display when results are being loaded
     */
    searchResultsGroupName?: string;

    /**
     * Text to display when there are no results (based on the current search criteria) 
     */
    searchNoResultsText?: string;

    /**
     * Text to to display when the search results are still loading.
     */
    searchResultsLoadingText?: string;

    /**
     * Optional method to perform a custom search. By default a simple string contains is applied to the names in the current set of items. 
     */
    onSearch?: (searchText: string, items: T[]) => T[] | Promise<T[]>;

    /**
     * Optional list of actions to render after all items and groups
     */
    actions?: IPickListAction[];

    /**
     * Groups of items to render.
     */
    groups?: IPickListGroup[];

    /**
     * Text or a component representing a rich text message to display when there are no items in the list
     */
    noItemsText?: (string | JSX.Element);

    /**
     * Optional delegate to return a custom text value to use for the hub title when the provided item is selected.
     * By default, the item's 'text' property is used (the same text used in the picker dropdown).
     * This can be leveraged if you want the text used in the hub title area to be different for a given item compared to what shows up in the picker dropdown.
     */
    getTitleTextForItem?: (item: T) => string;

    /**
     * Aria-Label for the picker
     */
    ariaLabel?: string;
}

/**
 * Props for the component rendering item picker provider source inside a Combo-like dropdown.
 */
export interface IItemPickerDropdownProps {

    /**
     * The item picker to render in a PickListDropdown component
     */
    provider: IItemPickerProvider<any>;

    /**
     * Class name for the root PickListDropdown component
     */
    className?: string;
    
    /**
     * Optional callback to access the IItemPickerDropdown interface. Use this instead of ref for accessing
     * the public methods and properties of the component.
     */
    componentRef?: (component: IItemPickerDropdown) => void;
    
    /**
     * Class name for the pick list shown in the dropdown
     */
    pickListClassName?: string;
    
    /**
     * MinWidth for the PickList used in the dropdown
     */
    minWidth?: number;
    
    /**
     * The tab index for the dropdown element
     */
    tabIndex?: number;

    /**
     * If true, do not show the icon on the root selected item (on the component that you click to open the dropdown)
     */
    hideSelectedItemIcon?: boolean;

    /**
     * Overrides the default (chevron) dropdown icon
     */
    dropdownIconProps?: IVssIconProps;

    /**
     * Optional callback which custom rendering of pick list element.
     */
    onRenderItem?: (item: IPickListItem) => JSX.Element;
}

export interface IItemPickerDropdown {
    pickListDropdown: IPickListDropdown;
}

/**
 * Props for the rendering the items in a pick list provider source in a list.
 */
export interface IItemPickListProps {
    
    /**
     * The item picker to render in a pick list
     */
    provider: IItemPickerProvider<any>;

    /**
     * Class name for the root pick list component
     */
    className?: string;

    /**
     * If true, use the same style as dropdown pick lists. By default, uses
     * the style common to standalone pick list controls.
     */
    useDropdownStyle?: boolean;
    
    /**
     * Optional callback to access the IItemPickList interface. Use this instead of ref for accessing
     * the public methods and properties of the component.
     */
    componentRef?: (component: IItemPickList) => void;

    /**
     * Optional callback which custom rendering of pick list element.
     */
    onRenderItem?: (item: IPickListItem) => JSX.Element;

    /**
     * Optional callback to determine whether the PickList should be rendered in full or virtualized.
     * Default behavior is virtualized which is suggested.
     */
    shouldVirtualize?: (itemsToRender: IPickListItem[]) => boolean;
}

export interface IItemPickList {
    pickList: IPickList;
}