import * as React from 'react';
import { IFilterBarProps } from '../../Components/FilterBar/FilterBar.Props';
import { IPivotBarAction, IPivotBarViewAction, IVssPivotBarActionProvider } from '../../Components/PivotBar/PivotBarAction.Props';
import { IFilter } from '../../Utilities/Filter/interfaces';
import { IObservableArray, IObservableValue } from '../../Utilities/Observable/interfaces';
import { IItemProvider } from '../../Utilities/ItemContribution/interfaces';
import { IViewOptions } from '../../Utilities/ViewOptions/interfaces';

/**
 * Interface for objects that provide PivotBar items (e.g. from contributions)
 */
export interface IVssPivotBarItemProvider extends IItemProvider<IVssContributedPivotBarItem> {
}

/**
 * Interface for contributed PivotBarItems to implement.
 */
export interface IVssContributedPivotBarItem {
    /**
     * Unique identifier (maps to itemKey on IPivotBarProps) 
     */
    id: string;

    /**
     * Text to display for the Pivot
     */
    text: string;

    /** 
     * True if the item should be disabled
     */
    disabled?: boolean;

    /**
     * Unique key for the PivotBarItem. For non-trusted contribution sources this might be ignored.
     */
    itemKey?: string;

    /**
     * Optional group key for the pivot item
     */
    groupKey?: string;

    /**
     * Optional order property. Used for sorting the pivot bar items when displaying in the pivot bar.
     */
    order?: number;

    /**
     * Render the pivot content into an element with the given CSS class.
     */
    render(className?: string): JSX.Element;

    /**
     * Providers to provide contributed commands.
     */
    commandProviders?: IVssPivotBarActionProvider[];

    /**
     * Providers to provide contributed view actions.
     */
    viewActionProviders?: IVssPivotBarActionProvider[];

    /**
     * Optional method to render this pivot's FilterBar. This overrides any FilterBar supplied by the PivotBar.
     */
    renderFilterBar?: () => React.ReactElement<IFilterBarProps> | undefined;

    /**
     * Url that the contributed pivot item points to.
     */
    url?: IObservableValue<string> | string;

    /**
     * Contribution that the pivot was created from. Not used by the PivotBar, but may be useful
     * for consumers.
     */
    sourceContribution?: any;
}

export const enum PivotBarFocusItem {
    /**
     * Commands section of the pivot command bar.
     */
    commands
}

/**
 * Basic information about a pivot item
 */
export interface IPivotBarItem {

    /**
     * Unique identifier for the pivot
     */
    itemKey: string;

    /**
     * Display name for the pivot
     */
    name: string;

    /**
     * Optional key for the group to display the pivot in
     */
    groupKey?: string;

    /**
     * Optional href for the pivot
     */
    url?: IObservableValue<string> | string;

    /**
     * For contributed pivot bar items, this is the original contributed item
     */
    contributedItem?: IVssContributedPivotBarItem;
}

export interface IPivotBar {
    /**
     * Sets focus to the pivot bar. If a particular focusItem is provided, will filter the focus() call
     * down to the appropriate sub-component.
     */
    focus(focusItem: PivotBarFocusItem): void;

    /**
     * Make the given pivot the active pivot. Returns true on success.
     */
    selectPivot(pivotKey: string): boolean;

    /**
     * Gets the currently selected pivot
     */
    getSelectedPivot(): IPivotBarItem;
}

export interface IPivotRenderingModeOptions {
    /**
     * Pivot rendering mode
     */
    mode: PivotRenderingMode;

    /**
     * Properties associated with the specified pivot rendering mode
     */
    props: IPivotRenderingModeProps;
}


export interface IPivotRenderingModeProps {
}

export interface IDropDownPivotRenderingModeProps extends IPivotRenderingModeProps {
    /**
     * Custom width for dropdown. By default, width of the input field is used.
     */
    dropdownWidth?: number;
}

export interface IVerticalPivotRenderingModeProps extends IPivotRenderingModeProps {
    /** 
     * Optional groups under which items will be rendered. If no groups are defined,
     * or an item specifies a group which does not exist, the item's link will be
     * rendered after all groups.
    */
    groups?: IPivotGroup[];
}

export interface IPivotGroup {
    /** 
     * Key which is referenced by items
    */
    key: string;
    /** 
     * Text rendered for the group
    */
    name: string;
    /** 
     * Order for the groups. If none provided, defaults to 100. Lowest rank renders first. 
    */
    rank?: number;
}

export const enum PivotRenderingMode {
    /**
     * Pivots will be rendered horizontally
     */
    HorizontalTabs,
    /**
     * Pivots will be rendered with DropDown control
     */
    DropDown,
    /**
     * Pivots will be rendered vertically, to the left of the command bar
     */
    VerticalTabs
}

export interface IPivotBarProps {
    /**
     * Optional pivot rendering mode options. By default, pivots will be rendered horizontally.
     */
    pivotRenderingModeOptions?: IPivotRenderingModeOptions;
    /**
     * Optional callback to access the IPivotBar interface. Use this instead of ref for accessing
     * the public methods and properties of the component.
     */
    componentRef?: (component: IPivotBar) => void;

    /**
     * The view options store updated by view actions.
     */
    viewOptions?: IViewOptions;

    /**
     * Optional root classname for the root PivotBar element.
     */
    className?: string;

    /**
     * The key of the initially selected pivot.
     */
    initialSelectedPivot?: string;

    /**
     * The key of the selected pivot item. If set, this will override the Pivot's selected item state.
     */
    selectedPivot?: string;

    /**
     * Called when the user requests to change the pivot. If this method doesn't return true, the
     * change will be blocked.
     */
    onBeforePivotChange?: (ev: React.MouseEvent<HTMLElement> | React.KeyboardEvent<HTMLElement>, pivotKey: string, url?: string) => boolean;

    /**
     * Callback issued when the selected pivot item was changed.
     */
    onPivotClicked?: (ev: React.MouseEvent<HTMLElement> | React.KeyboardEvent<HTMLElement>, pivotKey: string) => void;

    /**
     * Set of commands available on every pivot. The global commands are ordered before pivot-specific commands.
     */
    commands?: IPivotBarAction[] | IObservableArray<IPivotBarAction>;

    /**
     * Optional method to render this pivot's FilterBar. A FilterBar can be supplied as a child element of this PivotBar, or be returned by this delegate.
     */
    onRenderFilterBar?: () => React.ReactElement<IFilterBarProps>;

    /**
     * Optional callback that is invoked after the pivot bar has been rendered.
     */
    onRenderComplete?: () => void;

    /**
     * Optional value providing explicit control over where the filter bar is displayed:
     * If true, show the filter bar inline with the pivot commands and view actions.
     * If false, add a toggle button in the view actions which displays the filter bar in a new row
     * If undefined (default), shows inline only if there is a single FilterBar item and it is a Keyword filter item.
     */
    showFilterBarInline?: boolean;
    
    /**
     * The key for the view option that controls whether or not the filter bar should be displayed. If not passed,
     * then the state is kept internally, and not managed through the viewOptions.
     */
    showFilterBarViewOptionKey?: string;

    /**
     * Set of far actions available on every pivot. Global far actions are ordered after pivot-specific far actions.
     */
    viewActions?: IPivotBarViewAction[] | IObservableArray<IPivotBarViewAction>;

    /**
     * List of PivotBar item providers
     */
    pivotProviders?: IVssPivotBarItemProvider[];

    /**
     * The default filter to use for child filter bars and items.
     */
    filter?: IFilter;

    /**
     * Explicitly sets the visibility of the pivots (tabs). If false, render the commands and view actions inline with any inner (non-pivot) content.
     * If this prop is undefined (default), then the pivots will be shown if there is more than one pivot item.
     */
    showPivots?: boolean;

    /**
     * CSS class name to append to this pivot's filter bar
     */
    filterBarClassName?: string;

    /**
     * CSS class name to append to pivot bar content
     */
    pivotBarContentClassName?: string;

    /**
     * Whether PivotBar content should have data-is-scrollable set to true.
     * @defaultvalue true
     */
    isPivotBarContentScrollable?: boolean;
}