/// <reference types="react" />
import * as React from 'react';
import { BaseComponent } from 'office-ui-fabric-react/lib/Utilities';
import { IPivotBarProps, IPivotBar, PivotBarFocusItem, IPivotBarItem } from '../../Components/PivotBar/PivotBar.Props';
import { PivotBarItem } from '../../Components/PivotBar/PivotBarItem';
import { PivotBarItemDeselectionBehavior } from '../../Components/PivotBar/PivotBarItem.Props';
import { IPivotBarAction, IPivotBarViewAction, IVssPivotBarActionProvider } from '../../Components/PivotBar/PivotBarAction.Props';
import { IFilterBarProps } from '../../FilterBar';
import { IVssIconProps } from '../../VssIcon';
import { IObservableArray } from '../../Utilities/Observable';
import "./PivotBar.css";
/**
 * How long after load to wait (in milliseconds) to see if the selected pivot gets contributed
 * before giving up and selecting a different pivot instead.
 */
export declare const MAX_PIVOT_WAIT = 1000;
export interface IPivotBarState {
    selectedPivot?: string;
    items?: IPivotBarItemInternal[];
    commands?: IObservableArray<IPivotBarAction>[];
    viewActions?: IObservableArray<IPivotBarViewAction>[];
    showFilterBar?: boolean;
    hasAppliedFilter?: boolean;
}
/**
 * Internal structure used by PivotBar to keep track of its items (initial and contributed).
 *
 * Basically mirrors IPivotbarItemProps with the addition of id, child, and contribution.
 */
export interface IPivotBarItemInternal extends IPivotBarItem {
    id: string;
    order: number;
    ariaLabel?: string;
    badgeCount?: number;
    maxItemWidth?: number;
    iconProps?: IVssIconProps;
    commands?: IPivotBarAction[] | IObservableArray<IPivotBarAction>;
    viewActions?: IPivotBarViewAction[] | IObservableArray<IPivotBarViewAction>;
    child?: PivotBarItem;
    commandProviders?: IVssPivotBarActionProvider[];
    viewActionProviders?: IVssPivotBarActionProvider[];
    deselectionBehavior?: PivotBarItemDeselectionBehavior;
    onRenderFilterBar?: () => React.ReactElement<IFilterBarProps>;
}
export declare type RuntimeProps<T> = Readonly<T & {
    children?: React.ReactNode;
}>;
/**
 * A PivotBar combines a Pivot component (tabs and tabbed content) along with a set of
 * commands (rendered after the pivots) and view actions (rendered on the far right).
 */
export declare class PivotBar extends BaseComponent<IPivotBarProps, IPivotBarState> implements IPivotBar {
    private _id;
    private _commandBar;
    private _contributedPivots;
    private _subscribedToPivotObservables;
    private _actionsManager;
    private _contributedPivotWait;
    /** True if we have successfully found the selected pivot after load */
    private _pivotSelected;
    /** Remembers which pivot items have been rendered */
    private _pivotItemsRendered;
    constructor(props: IPivotBarProps);
    focus(focusItem: PivotBarFocusItem): void;
    selectPivot(pivotKey: string): boolean;
    /**
     * Gets the currently selected pivot
     */
    getSelectedPivot(): IPivotBarItem;
    /**
     * This gets executed when a container component re-renders PivotBar.
     * @param nextProps New props.
     */
    componentWillReceiveProps(nextProps: RuntimeProps<IPivotBarProps>): void;
    componentDidMount(): void;
    componentDidUpdate(): void;
    componentWillUnmount(): void;
    render(): JSX.Element;
    private _renderPivotItems(pivotRenderingModeOptions);
    private _renderDropdown(props);
    private _onDropDownSelected;
    private _renderHorizontalTabs();
    private _renderVerticalTabs();
    private _makeNavLinkFromPivotBarItem(item);
    private _renderPivotItem;
    private _renderPivotItemContent(item);
    private _renderCommandBar(filterBar, showFilterAction, prependSeparator);
    private _renderContent();
    private _onShowFilterBarClicked;
    private _onFilterApplied;
    private _onContributedPivotsChanged;
    private _onViewOptionChanged;
    private _makeIPivotBarItemFromContribution;
    private _makePivotBarItemId(child);
    private _makeContributedPivotBarItemId(item, index);
    private _getPivotBarItemByItemKey(itemKey, items?);
    /**
     * Extracts the pivot items from the child elements.
     * @param props PivotBar properties.
     */
    private _getPivotBarItems(props);
    private _setupPivotObservables(props);
    private _unsubscribeFromPivotObservables();
    private _subscribeToPivotObservables();
    private _getFilterBarSettings(children);
    private _getHubTileRegionIndex(children);
    /**
     * Determines whether the key exists in the pivot items or not.
     */
    private _isKeyValid(itemKey, items?);
    private _buildInitialPivotState(props?, items?, resetActions?);
    private _buildNewPivotState(props, selectedPivotItemKey, items?, resetActions?);
    /**
     * Handles the onClick event on the pivot button.
     */
    private _onPivotClick(itemKey, ev);
    /**
     * Handle the onKeyPress event on the pivot button.
     */
    private _onKeyPress(itemKey, ev);
    /**
     * Updates the state with the new selected pivot.
     */
    private _updateSelectedItem(itemKey, ev);
}
