import { IChoiceGroupOption } from 'office-ui-fabric-react/lib/ChoiceGroup';

import { IUserAction } from '../../Utilities/IUserAction';
import { IItemProvider } from '../../Utilities/ItemContribution/interfaces';
import { IObservableValue } from '../../Utilities/Observable/interfaces';
import { IVssIconProps } from '../../Components/VssIcon/VssIcon.Props';

/**
 * Represents where a particular view action should be rendered within the pivot bar
 */
export const enum PivotBarViewActionArea {
    /**
     * By deafult, the view action will be rendered after the View Options built-in action
     */
    default,
    /**
     * The view action will be be rendered before the View Options built-in action. In order for
     * this value to be honored, the action must almost be marked as important.
     */
    beforeViewOptions,
    /**
     * The view action will be rendered at a group to the right after other view actions, view options, and filter action.
     */
    farRight,
}

export const enum PivotBarViewActionType {
    /**
     * Command action where it displays a text and an icon (optional).
     * Executing this action will trigger an event or navigate to the specified href.
     */
    Command,

    /**
     * An action switching between on and off states.
     * Executing this action will trigger viewOption change with a boolean value true being action is on.
     */
    OnOff,

    /**
     * Action which lets you select one option from two or more options.
     * Executing this action will trigger viewOption change with a string value being the key of the selected option.
     */
    ChoiceGroup,

    /**
     * No action with only a text and icon (optional).
     */
    Text,

    /**
     * An action to set a numeric value. It provides a visual indication of adjustable content, 
     * as well as the current setting in the total range of content.
     */
    Slider
}

/**
 * Base PivotBar action.
 */
export interface IPivotBarAction extends IUserAction {
    /**
     * Determines whether the action is always visible or not.
     *
     * If the number of max important items is exceeded for PivotBar, an action can go
     * to overflow section even though it is marked as important.
     * @defaultvalue false
     */
    important?: boolean;

    /**
     * Icon to display for the action.
     */
    iconProps?: IVssIconProps;

    /**
     * Child PivotBar action.
     */
    children?: IPivotBarAction[];

    /**
     * Determines whether this action is disabled or not.
     * @defaultvalue false
     */
    disabled?: boolean;

    /**
     * Determines if this action is a seperator, which will set the itemType to ContextualMenuItemType.Divider.
     */
    separator?: boolean;
}

/**
 * PivotBar view action.
 */
export interface IPivotBarViewAction extends IPivotBarAction {
    /**
     * Type of the view action.
     * @defaultvalue "Default"
     */
    actionType?: PivotBarViewActionType;

    /**
     * Properties for the specified view action type.
     */
    actionProps?: IPivotBarViewActionProps;

    /**
     * Child PivotBar view actions.
     */
    children?: IPivotBarViewAction[];

    /**
     * Where the view action should be rendered in relation to the built-in 'View Options' action
     */
    viewActionRenderArea?: PivotBarViewActionArea;
}

export interface IVssPivotBarActionProvider extends IItemProvider<IPivotBarAction> {
}

export interface IPivotBarViewActionProps {
    /**
     * Class to be applied to the root component of the action
     */
    className?: string;
}

export interface IOnOffViewActionProps extends IPivotBarViewActionProps {
    /**
     * Text for screen reader to announce when the state is on.
     */
    onAriaLabel?: string;

    /**
     * Text for screen reader to announce when the state is off.
     */
    offAriaLabel?: string;

    /**
     * Icon for on state.
     */
    onIconProps?: IVssIconProps;

    /**
     * Icon for off state.
     */
    offIconProps?: IVssIconProps;

    /**
     * Show text On/Off text or not.
     * @defaultvalue true
     */
    showText?: boolean;

    /**
     * Text to display when toggle is ON.
     */
    onText?: string;

    /**
     * Text to display when toggle is OFF.
     */
    offText?: string;
}

export interface IChoiceGroupViewActionOption extends IChoiceGroupOption {
    /**
      * Optional accessibility label (aria-label) attribute that will be stamped on to the element.
      */
    ariaLabel?: string;
}

export interface IChoiceGroupViewActionProps extends IPivotBarViewActionProps {
    /**
     * The options for the choice group.
     */
    options: IChoiceGroupViewActionOption[];

    /** 
     * The default selected key for this choice group, if the linked view option value is null
     */
    defaultSelectedKey?: string;

    /**
     * Optional accessibility label for drop button which is used when the action is visible on the bar.
     */
    dropButtonAriaLabel?: string;
}

export interface ITextViewActionProps extends IPivotBarViewActionProps {
    /**
     * Text value of the view action.
     */
    text: IObservableValue<string> | string;
}

export interface ISliderViewActionProps extends IPivotBarViewActionProps {
    /**
     * @defaultvalue 0
     * The minimum value of the Slider.
     */
    minValue?: number;

    /**
     * The maximum value of the Slider.
     * @defaultvalue 10
     */
    maxValue?: number;

    /**
     * The different between the two adjacent values of the Slider.
     * @defaultvalue 1
     */
    step?: number;
}