import * as React from 'react';
import { IBaseProps } from "office-ui-fabric-react/lib/Utilities";
import { IPivotBarAction, IPivotBarViewAction, IVssPivotBarActionProvider } from '../../Components/PivotBar/PivotBarAction.Props';
import { IVssIconProps } from '../../VssIcon';
import { IFilterBarProps } from '../../Components/FilterBar/FilterBar.Props';
import { IObservableArray, IObservableValue } from '../../Utilities/Observable';

/**
 * Rendering behavior of pivot bar item.
 */
export const enum PivotBarItemDeselectionBehavior {
    /**
     * Removes (unmounts) the component when the pivot is deselected.
     */
    Remove,

    /**
     * Hides the component when the pivot is deselected.
     * This enables reduction of the performance overhead caused by mounting.
     */
    Hide
}

export interface IPivotBarItemProps extends React.HTMLProps<HTMLDivElement>, IBaseProps {
    /**
     * The text displayed of each pivot link.
     */
    name: string;

    /**
     * A key to uniquely identify a pivot item.
     *
     * Note: The 'key' from react props cannot be used inside component.
     */
    itemKey: string;

    /**
     * Optional group key for the pivot item
     */
    groupKey?: string;

    /**
     * The aria label of each pivot link which will read by screen reader instead of name.
     */
    ariaLabel?: string;

    /**
      * Optional item count that gets displayed just after the linkText.
      */
    badgeCount?: number;

    /**
     * An optional icon to show next to the pivot name.
     */
    iconProps?: IVssIconProps;

    /**
     * If specified, use an anchor tag in the pivot tab pointing to the specified URL
     * This allows ctrl-click, right-click, etc. to open in new tab, window, etc.
     */
    url?: IObservableValue<string> | string;

    /**
     * Set of commands available for this pivot.
     * Pivot-specific commands are ordered after global commands.
     */
    commands?: IPivotBarAction[] | IObservableArray<IPivotBarAction>;

    /**
     * Providers to provide contributed commands.
     */
    commandProviders?: IVssPivotBarActionProvider[];

    /**
     * Set of far actions available for this pivot.
     * Pivot-specific far actions are ordered after global far actions.
     */
    viewActions?: IPivotBarViewAction[] | IObservableArray<IPivotBarViewAction>;

    /**
     * Providers to provide contributed view actions.
     */
    viewActionProviders?: IVssPivotBarActionProvider[];

    /**
     * CSS class name for the pivot item container element.
     */
    className?: string;

    /**
     * CSS class name for the pivot bar header
     */
    headerClassName?: string;

    /**
     * Optional value for max width of this pivot item name. 
     * If name width is larger than this value, name is cut and full value is displayed as a tooltip.
     * @defaultvalue 340px
     */
    maxItemWidth?: number;

    /**
     * Optional setting to change the render behavior of the pivot content.
     * @defaultvalue PivotBarItemDeselectionBehavior.Remove
     */
    deselectionBehavior?: PivotBarItemDeselectionBehavior;
    
    /**
     * Optional method to render this pivot's FilterBar. This overrides any FilterBar supplied by the PivotBar.
     */
    onRenderFilterBar?: () => React.ReactElement<IFilterBarProps> | undefined;

    /**
     * Allows a Hub Component to define multiple PivotItems but conditionally decide
     * whether or not the name item should actually appear in the PivotBar.
     */
    hidden?: boolean;
}