import { Observable } from '../Observable';
import { IFilter, IFilterItemState, IFilterOptions, IFilterState } from "./interfaces";
export declare const FILTER_CHANGE_EVENT = "filter-changed";
export declare const FILTER_APPLIED_EVENT = "filter-applied";
export declare const FILTER_RESET_EVENT = "reset-filters";
/**
 * Store for a set of filter values
 *
 * Events:
 *
 *  FILTER_CHANGE_EVENT: IFilterState
 *  Fired whenever a filter value changes. The event object contains the changed items.
 *  When using explicit apply-mode, this event is fired when the filter is changed (i.e. by user action)
 *  but not yet applied.
 *
 *  FILTER_APPLIED_EVENT: IFilterState
 *  Fired when new filter changes have been applied. The event object contains the changed items.
 *  When not using explicit apply-mode, this event is fired on every state change
 */
export declare class Filter implements IFilter {
    private observable;
    private defaultState;
    private currentState;
    private appliedState;
    private applyMode;
    private customValueComparers;
    /**
     * Create a new Filter store
     * @param options Options for the filter store
     */
    constructor(options?: IFilterOptions, observable?: Observable<IFilterState>);
    subscribe(observer: (value: IFilterState, action: string) => void, action?: string): void;
    unsubscribe(observer: (value: IFilterState, action: string) => void, action?: string): void;
    /**
     * Gets a dictionary containing the current values of all filter items
     */
    getState(): IFilterState;
    /**
     * Gets a dictionary containing the applied values of all filter items.
     * When useApplyMode is false, this always matches the value returned by getState.
     */
    getAppliedState(): IFilterState;
    /**
     * Gets a dictionary containing the default values of all filter items
     */
    getDefaultState(): IFilterState;
    /**
     * Update what the filter considers as its default state
     * @param defaultState The new default state
     */
    setDefaultState(defaultState: IFilterState): void;
    /**
     * Sets the current values of all filter items
     * @param state Dictionary of all current filter item values
     * @param supressChangeEvent If true, don't invoke the onFilterChanged callback at this time
     */
    setState(state: IFilterState, supressChangeEvent?: boolean): void;
    /**
     * Gets the value of the specified filter item
     * @param key Filter item key
     */
    getFilterItemState(key: string): IFilterItemState;
    /**
     * Gets the applied value of the specified filter item. When applyMode is false,
     * this is equivalent to getFilterItemState.
     * @param key Filter item key
     */
    getAppliedFilterItemState(key: string): IFilterItemState;
    /**
     * Gets the value property for the filter item with the given key.
     * @param key The filter item's key
     */
    getFilterItemValue<T>(key: string): T;
    /**
     * Sets the value of the specified filter item
     * @param key Filter item key
     * @param value Filter value
     */
    setFilterItemState(key: string, value: IFilterItemState): void;
    /**
     * Resets the filter values to their default state
     */
    reset(): void;
    applyChanges(): void;
    usesApplyMode(): boolean;
    hasChangesToApply(): boolean;
    hasChangesToReset(): boolean;
    statesAreEqual(state1: IFilterState, state2: IFilterState): boolean;
    filterItemStatesAreEqual(item: string, state1: IFilterItemState, state2: IFilterItemState): boolean;
    private _triggerStateChange(changedState);
    private _raiseEventAndCallListeners(eventName, changedState);
    private _checkStateEquality(state1, state2);
    private _checkFilterItemStateEquality(item, item1State, item2State);
    private _checkValueEquality(key, item1, item2);
}
