import { IObservable } from "../Observable/interfaces";

export const enum FilterOperatorType {
    /**
     * The filter's value should be treated as having the AND operator
     * e.g. If the value is ['a', 'c', 'd'] any item which does not contain
     * 'a' AND 'c' AND 'd' should not be included
     */
    and = "and",
    /**
     * The filter's value should be treated as having the OR operator
     * e.g. If the value is ['a', 'c', 'd'] any item which contains
     * 'a' OR 'c' OR 'd' should be included
     */
    or = "or",
}

/**
 * Dictionary of filter item values
 */
export interface IFilterState {
    /**
     * Maps a key to an IFilterItemState
     */
    [key: string]: IFilterItemState;
}

/**
 * Contains the filter item's value and an optional operator
 */
export interface IFilterItemState {
    /**
     * Current value of a filter item
     */
    value: any;

    /**
     * Operator currently being used in the filter item.
     */
    operator?: FilterOperatorType;
}

/**
 * Store for a set of filter values
 */
export interface IFilter extends IObservable<IFilterState> {

    /**
     * Gets the value of the specified filter item
     * @param key Filter item key
     */
    getFilterItemState: (key: string) => IFilterItemState;

    /**
     * Gets the value property for the filter item with the given key.
     * @param key The filter item's key
     */
    getFilterItemValue<T>(key: string): T;

    /** 
     * Sets the value of the specified filter item
     * @param key Filter item key
     * @param value Filter value
     */
    setFilterItemState: (key: string, value: IFilterItemState) => void;

    /**
     * Update what the filter considers as its default state
     * @param defaultState The new default state
     */
    setDefaultState: (defaultState: IFilterState) => void;

    /** 
     * Gets a dictionary containing the default values of all filter items
     */
    getDefaultState: () => IFilterState;

    /** 
     * Gets a dictionary containing the current values of all filter items
     */
    getState: () => IFilterState;

    /** 
     * Gets a dictionary containing the applied values of all filter items.
     * When useApplyMode is false, this always matches the value returned by getState.
     */
    getAppliedState(): IFilterState;

    /** 
     * Sets the current values of all filter items
     * @param state Dictionary of all current filter item values
     * @param supressChangeEvent If true, don't invoke the onFilterChanged callback at this time
     */
    setState: (state: IFilterState, suppressChangeEvent?: boolean) => void;

    /**
     * Resets the filter values to their default state
     */
    reset: () => void;

    /**
     * Updates the filter's applied state to match the current state - used for scenarios
     * where the consumer of the filter does not want to be notified on every change to the filter,
     * only when the user specifices that changes should be applied.
     */
    applyChanges: () => void;

    /**
     * If true, indicates that updates to this filter should not be applied immediately, but
     * rather only after an "apply" operation is performed (e.g. "enter" press or click "apply")
     */
    usesApplyMode: () => boolean;

    /**
     * Whether or not the applied state matches the current state.
     */
    hasChangesToApply: () => boolean;

    /**
     * Whether or not the default state matches the current state.
     */
    hasChangesToReset: () => boolean;

    /**
     * Whether or not the two specified states are considered equal.
     */
    statesAreEqual: (state1: IFilterState, state2: IFilterState) => boolean;

    /**
     * Whether or not the two specified states of a filterBarItem are considered equal.
     */
    filterItemStatesAreEqual: (item: string, state1: IFilterItemState, state2: IFilterItemState) => boolean;
}

/**
 * Options used when creating a filter
 */
export interface IFilterOptions {
    /**
     * Default state of the filter. When reset, the filter is restored to these values.
     */
    defaultState?: IFilterState;

    /**
     * If true, indicates that updates to this filter should not be applied immediately, but
     * rather only after an "apply" operation is performed (e.g. "enter" press or click "apply")
     */
    useApplyMode?: boolean;

    /**
     * Optional comparer functions to disentangle the displayed value from the internally maintained value.
     * For example, this could be useful in a TextFilterBarItem where the user inputs a number and you don't want to treat "0" and "0.0" as different filter values.
     */
    customValueComparers?: { [key: string]: (a: any, b: any) => boolean };
}