import { IObservableValue } from './Observable';
import { IViewState, ViewState } from './ViewState';
import { IFilter, IFilterOptions, IFilterState } from './Filter';
import { IViewOptions, IViewOptionsProps, IViewOptionsValues } from './ViewOptions';
/**
 * The keys for common view options known by the Hub component
 */
export declare namespace HubViewOptionKeys {
    /**
     * View option controlling whether or not the hub filter bar is being displayed.
     */
    const showFilterBar = "showFilters";
    /**
     * View option controlling whether or not the page is in full-screen mode
     */
    const fullScreen = "fullScreen";
}
/**
 * Names of observable events triggered by hub view state
 */
export declare namespace HubViewStateEventNames {
    /**
     * Event fired when the pivot is being changed. Consumers can listen for this
     * event and modify other view state values. This event is not fired when
     * the selected pivot value is changed during the processing of another view-state's event.
     */
    const pivotChanging = "pivotChanging";
    /**
     * Event fired when the applied filter is being changed. Consumers can listen for this
     * event and modify other view state values. This event is not fired when
     * the filter is changed during the processing of another view-state's event.
     */
    const filterChanging = "filterChanging";
    /**
     * Event fired when a view option value is being changed. Consumers can listen for this
     * event and modify other view state values. This event is not fired when
     * a view option value is changed during the processing of another view-state's event.
     */
    const viewOptionsChanging = "viewOptionsChanging";
}
/**
 * Store for the current view of a hub (selected pivot, filters, view options, etc.)
 */
export interface IHubViewState extends IViewState<IViewOptionsValues | IFilterState | string> {
    /**
     * The key of the currently selected hub pivot
     */
    selectedPivot: IObservableValue<string>;
    /**
     * Filter applied to the hub
     */
    filter: IFilter;
    /**
     * View options used by the hub
     */
    viewOptions: IViewOptions;
    /**
     * Called by the Hub before the user switches pivots, return false to block the change.
     */
    canSwitchPivots(targetPivot: string): boolean;
}
/**
 * Initialization options for HubViewState
 */
export interface IHubViewStateOptions {
    /**
     * Filter to use for the hub
     */
    filterOptions?: IFilterOptions;
    /**
     * View options used by the hub
     */
    viewOptions?: IViewOptionsProps;
    /**
     * The key of the default pivot. The selected pivot is set to this value initially.
     */
    defaultPivot?: string;
}
/**
 * Store for the current view of a hub (selected pivot, filters, view options, etc.)
 */
export declare class HubViewState extends ViewState<IViewOptionsValues | IFilterState | string> implements IHubViewState {
    private _viewOptions;
    private _filter;
    private _selectedPivot;
    constructor(options?: IHubViewStateOptions);
    readonly selectedPivot: IObservableValue<string>;
    readonly filter: IFilter;
    readonly viewOptions: IViewOptions;
    canSwitchPivots(targetPivot: string): boolean;
}
