import { IObservable, IObservableLikeValue, IObservableValue } from "./interfaces";
/**
 * An Observable implementation that will track a set of subscribers and supports
 * notifications when the underlying system changes.
 */
export declare class Observable<T> implements IObservable<T> {
    private observers;
    private events?;
    /**
     * notify is used to send the event to all subscribers that have signed up for this events
     * action. This means they have subscribed directly to this action, or to all actions.
     * If the caller requested the event be persisted the event will be fired in order to new
     * subscribers as well when they subscribe.
     *
     * @param event - The object that represents the event that has occurred.
     *
     * @param persistEvent - Optional value that determines if all future subscribers will
     *  recieve the event as well.
     */
    notify(value: T, action: string, persistEvent?: boolean): void;
    subscribe(observer: (value: T, action: string) => void, action?: string): void;
    unsubscribe(observer: (value: T, action: string) => void, action?: string): void;
}
export declare class ObservableValue<T> extends Observable<T> implements IObservableValue<T> {
    private v;
    constructor(value?: T);
    value: T;
}
export declare namespace ObservableLike {
    /**
     * Check whether the specified object is an observable or not.
     *
     * @param observableLike Object to perform observable check.
     */
    function isObservable<T>(observableLike: IObservable<T> | any): boolean;
    /**
     * Gets the value of the specified observable like. If not observable, returns the passed argument.
     *
     * @param observableLike Object to get the value.
     * @returns Observable value or the observable like itself.
     */
    function getValue<T>(observableLike: IObservableLikeValue<T>): T;
    /**
     * Subscribes to the specified object if it is an observable.
     *
     * @param observableLike Object to subscribe its value change if applicable.
     * @param observer Delegate to be executed when the underlying data changes.
     * @param action Optional argument that allows the consumer to supply a action
     *  with the delegate. If the action is supplied only those actions are delievered,
     *  while all actions are delivered is no action is supplied.
     */
    function subscribe<T>(observableLike: IObservable<T> | any, observer: (value: T, action: string) => void, action?: string): void;
    /**
     * Unsubscribes from the specified object if it is an observable.
     *
     * @param observableLike Object to subscribe its value change if applicable.
     * @param observer Delegate to be executed when the underlying data changes.
     * @param action Optional argument that allows the consumer to supply a action
     *  with the delegate. If the action is supplied only those actions are delievered,
     *  while all actions are delivered is no action is supplied.
     */
    function unsubscribe<T>(observableLike: IObservable<T> | any, observer: (value: T, action: string) => void, action?: string): void;
}
/**
 * All ObservableArray events will have an action associated with them.
 */
export interface IObservableArrayEventArgs<T> {
    /**
     * Items added to ObservableArray.
     */
    addedItems?: T[];
    /**
     * Items removed from ObservableArray.
     */
    removedItems?: T[];
}
/**
 * WARNING: A copy of this interface also exists in WebPlatform\Web\extensions\vss-web\vss-platform\Context.ts
 * Please make sure that you keep both instances in sync.
 *
 * An Observable array is used to track an array of items and offer notifications
 * for consumers when the array has changed.
 *
 * EventTypes:
 *  push - { addedItems }
 *  pop - { removedItems}
 *  splice - { addedItems, removedItems }
 */
export interface IObservableArray<T> extends IObservable<IObservableArrayEventArgs<T>> {
    /**
     * Appends new elements to an array, and returns the new length of the array.
     *
     * @param items - new elements of the ObservableArray.
     *
     * @returns - number of the newly inserted items.
     */
    push: (...items: T[]) => number;
    /**
     * Removes the last element from an array and returns it.
     *
     * @returns - removed element or undefined if ObservableArray has no items.
     */
    pop: () => T | undefined;
    /**
     * Removes elements from an array and, if necessary, inserts new elements in their place, returning the deleted elements.
     *
     * @param start - Zero-based location in the array from which to start removing elements.
     *
     * @param deleteCount - Number of elements to remove.
     *
     * @param items - Elements to insert into the array in place of the deleted elements.
     *
     * @returns - deleted elements.
     */
    splice: (start: number, deleteCount: number, ...items: T[]) => T[];
    /**
      * Returns the index of the first occurrence of a value in ObservableArray.

      * @param searchElement  - The value to locate in the array.
      *
      * @param fromIndex  - The array index at which to begin the search. If fromIndex is omitted, the search starts at index 0.
      *
      * @returns The index of the searched value. If not found, returns 0.
      */
    indexOf(searchElement: T, fromIndex?: number): number;
    /**
     * Gets or sets all the items in ObservableArray.
     */
    items: T[];
    /**
     * Gets the number of the items in ObservableArray.
     */
    length: number;
}
/**
 * WARNING: A copy of this interface also exists in WebPlatform\Web\extensions\vss-web\vss-platform\Context.ts
 * Please make sure that you keep both instances in sync.
 *
 * EventTypes:
 *  push - { addedItems }
 *  pop - { removedItems}
 *  splice - { addedItems, removedItems }
 */
export declare class ObservableArray<T> extends Observable<IObservableArrayEventArgs<T>> implements IObservableArray<T> {
    private internalItems;
    constructor(items?: T[]);
    push(...items: T[]): number;
    pop(): T | undefined;
    splice(start: number, deleteCount: number, ...itemsToAdd: T[]): T[];
    indexOf(item: T, fromIndex?: number): number;
    items: T[];
    readonly length: number;
}
