import { IObservable, IObservableValue, Observable } from './Observable';
import { IFilterState, IFilterOptions, Filter } from './Filter';
import { IViewOptionsProps, ViewOptions } from './ViewOptions';
/**
 * Names of events fired by view state
 */
export declare namespace ViewStateEventNames {
    /**
     * One or more members of the view state is changing
     */
    const viewStateChanging = "viewStateChanging";
    /**
     * One or more members of the view state was changed.
     */
    const viewStateChanged = "viewStateChanged";
}
/**
 * Event object sent in view state changing and view state changed events.
 */
export interface IViewStateChangeEvent<T> {
    /**
     * Name of the view-state event that triggered the view-state changing/changed events
     */
    eventName: string;
    /**
     * The value of view-state event that triggered teh view-state changing/changed events
     */
    eventValue: T;
}
/**
 * Store for properties of a view
 */
export interface IViewState<T> extends IObservable<T | IViewStateChangeEvent<T>> {
}
/**
 * Delegate to notify a view-state one of its values is chaning.
 */
export interface IViewStateNotifyDelegate<T> {
    /**
     * Method to notify view-state listeners that a state value is changing.
     *
     * @param eventValue The value of the state that was changed
     * @param viewStateEventName The name of the state-change event to fire
     * @param observable The observable that is firing the event
     * @param observableEventName Name of the specific event that should be triggered on the observable
     * @param observableNotify Method to directly notify the observable's consumers
     */
    (eventValue: T, viewStateEventName: string, observable: IObservable<T | IViewStateChangeEvent<T>>, observableEventName: string, observableNotify: {
        (value: T, action: string): void;
    }): void;
}
/**
 * Store for properties of a view
 */
export declare class ViewState<T> implements IViewState<T> {
    private observable;
    private changingEvents;
    subscribe(observer: (value: T, action: string) => void, action?: string): void;
    unsubscribe(observer: (value: T, action: string) => void, action?: string): void;
    /**
     * Notify listeners that a state value is changing. The sequence of events goes like this:
     *
     * 1. The 'viewStateChanging' event is fired
     * 2. The 'viewStateEventName' event is fired, letting consumers update state in reaction to the change.
     *    While these listeners are processing, individual state change events are suppresed.
     * 3. The observable event specified here is triggered
     * 4. The events from any state changed as part of the event listeners in step 2 are triggered.
     * 5. The 'viewStateChanged' event is fired
     *
     * @param eventValue The value of the state that was changed
     * @param viewStateEventName The name of the state-change event to fire
     * @param observable The observable that is firing the event
     * @param observableEventName Name of the specific event that should be triggered on the observable
     * @param observableNotify Method to directly notify the observable's consumers
     */
    protected notify(eventValue: T, viewStateEventName: string, observable: IObservable<T>, observableEventName: string, observableNotify: {
        (value: T, action: string): void;
    }): void;
}
/**
 * An observable value which is used in a view state
 */
export interface IViewStateObservable<T> extends IObservable<T> {
    /**
     * Method called when more than one of a particular event is triggered
     * for a single view-state-change operation. The default behavior is that the
     * most-recent value is taken.
     */
    mergeChangeEvents: (firstEventValue: T, nextEventValue: T) => T;
}
/**
 * An observable value which is used in a view state
 */
export declare class ViewStateObservable<T> extends Observable<T> implements IViewStateObservable<T> {
    private notifyViewStateDelegate;
    private viewStateEventName;
    private mergeEventsAsObjects;
    /**
     * Create a new view state observable
     *
     * @param notifyViewStateDelegate The view state's notify method
     * @param viewStateChangeEventName The name of the view state change event
     */
    constructor(notifyViewStateDelegate: IViewStateNotifyDelegate<T>, viewStateChangeEventName: string, mergeEventsAsObjects?: boolean, bypassCallback?: (value: T, action: string) => boolean);
    notify(value: T, action: string, persistEvent?: boolean): void;
    protected shouldBypassViewState(value: T, action: string, persistEvent?: boolean): boolean;
    mergeChangeEvents(firstEventValue: T, nextEventValue: T): T;
}
/**
 * An observable value which is used in a view state
 */
export declare class ViewStateObservableValue<T> extends ViewStateObservable<T> implements IObservableValue<T> {
    private v;
    constructor(notifyViewStateDelegate: IViewStateNotifyDelegate<T>, viewStateChangeEventName: string, value?: T);
    value: T;
}
/**
 * Filter used within a view state
 */
export declare class ViewStateFilter extends Filter {
    constructor(notifyViewStateDelegate: IViewStateNotifyDelegate<IFilterState>, viewStateChangeEventName: string, options?: IFilterOptions);
}
/**
 * ViewOptions used within a view state
 */
export declare class ViewStateOptions extends ViewOptions {
    constructor(notifyViewStateDelegate: IViewStateNotifyDelegate<IFilterState>, viewStateChangeEventName: string, options?: IViewOptionsProps);
}
